import { injectable } from 'inversify';
import { JsonRpcServer, Emitter, Event } from "@theia/core";

export namespace ISloggerServerTypes {
    
    export enum NotifcationData {
        OK = 0,
        ERROR = 1,
        STOPPED = 2
    }

    export class ImportFileResult {
        resultCode: ImportFileResultCode;
    }

    export enum ImportFileResultCode {
        OK = 0,
        UNKNOWN_FAILURE = 1
    }

    export class ChangeQueryResult {
    }

    export class GetEventsFromToResult {
        events: Events[];
    }

    export class Events {
        rowid: number;
        timestamp: number;
        data: Array<[string, string]>;;
    }

    export class ClearDataResult {
    }

    export class OnQueryChangedResult {
    }

    export class OnQueryChangedNotification {
        resultSize: number;
        notifcationData?: NotifcationData;
    }

    export class OnImportFileResult {
    }

    export class OnImportFileNotification {
        numberEvents: number;
        distinctFields: string[];
        notifcationData?: NotifcationData;
    }

}

export const SLOGGER_SERVICE_WS_PATH = '/services/slogger-service';
export const ISloggerServer = Symbol('ISloggerServer');

export interface ISloggerServer extends JsonRpcServer<ISloggerClient> {
    
    importFile(path: string): Promise<ISloggerServerTypes.ImportFileResult>;

    changeQuery(path: string, query: string): Promise<ISloggerServerTypes.ChangeQueryResult>;

    getEventsFromTo(fromIndex: number, toIndex: number): Promise<ISloggerServerTypes.GetEventsFromToResult>;

    clearData(): Promise<ISloggerServerTypes.ClearDataResult>;

    registerOnQueryChanged(): void;

    registerOnImportFile(): void;

}

export const ISloggerClient = Symbol('ISloggerClient');

export interface ISloggerClient {

    onQueryChanged(event: ISloggerServerTypes.OnQueryChangedNotification): void;

    onImportFile(event: ISloggerServerTypes.OnImportFileNotification): void;

}

@injectable()
export class DispatchingSLoggerClient implements ISloggerClient {

    readonly clients = new Set<ISloggerClient>();

    constructor() {
        console.log("**** DispatchingSLoggerClient.constructor")
    }

    push(client: ISloggerClient) {
        console.log("**** DispatchingSLoggerClient.push ");
        this.clients.add(client);
        console.log("**** DispatchingSLoggerClient.push attached Clients: "+this.clients.size);
    }

    remove(client: ISloggerClient) {
        console.log("**** DispatchingSLoggerClient.remove ");
        this.clients.delete(client);
    }

    onQueryChanged(event: ISloggerServerTypes.OnQueryChangedNotification): void {
        console.log("**** DispatchingSLoggerClient.onQueryChanged")
        this.clients.forEach(client => client.onQueryChanged(event));
    }

    onImportFile(event: ISloggerServerTypes.OnImportFileNotification): void {
        console.log("**** DispatchingSLoggerClient.onImportFile")
        this.clients.forEach(client => client.onImportFile(event));
    }
    
}

@injectable()
export class SloggerServerWatcher {
   
    private onImportFileEmitter = new Emitter<ISloggerServerTypes.OnImportFileNotification>();
    private onQueryChangedEmitter = new Emitter<ISloggerServerTypes.OnQueryChangedNotification>();

    getSloggerClient(): ISloggerClient {

        const importFileEmitter = this.onImportFileEmitter;
        const queryChangedEmiiter = this.onQueryChangedEmitter;

        console.log("**** Now creating a new SloggerWatcherClient");
        return {

            onImportFile(event: ISloggerServerTypes.OnImportFileNotification): void {
                importFileEmitter.fire(event);
            },

            onQueryChanged(event: ISloggerServerTypes.OnQueryChangedNotification): void {
                queryChangedEmiiter.fire(event);
            }
        }
    }

    get onImportFile(): Event<ISloggerServerTypes.OnImportFileNotification> {
        return this.onImportFileEmitter.event;
    }

    get onQueryChanged(): Event<ISloggerServerTypes.OnQueryChangedNotification> {
        return this.onQueryChangedEmitter.event;
    }
}