import { injectable, inject, postConstruct } from 'inversify';
import { ISloggerServer, ISloggerClient, ISloggerServerTypes } from "../common/slogger-protocol";
import { ImportFileRequest, ImportFileResponse, GetEventsFromToRequest, GetEventsFromToResponse, QueryRequest, QueryResponse, ClearDataRequest, ClearDataResponse, OnQueryChangedRequest, OnQueryChangedNotification, OnImportFileRequest, OnImportFileNotification } from '../api/slogger_service_pb';
import { SLoggerServiceClient } from '../api/slogger_service_grpc_pb';
import { IpcMessageType } from '../common/trace-protocol';

import * as grpc from '@grpc/grpc-js';
import { TracerNode } from './slogger-log4js';

@injectable()
export class SloggerRpcBridge implements ISloggerServer {

    protected client: ISloggerClient | undefined;

    readonly grpcProxy: SLoggerServiceClient;
    @inject(TracerNode) protected readonly logger: TracerNode;
    
    constructor () {
        this.grpcProxy = new SLoggerServiceClient("localhost:50051", grpc.credentials.createInsecure());
    }

    @postConstruct()
    init(): void {
        this.registerOnImportFile();
        this.registerOnQueryChanged();
    }

    importFile(path: string): Promise<ISloggerServerTypes.ImportFileResult> {

        let self = this;

        return new Promise( function (resolve, reject) {

            const request = new ImportFileRequest();
            request.setPath(path);

            self.logger.logIpcCall(0, IpcMessageType.Request, "slogger-service.importFile", request.toObject());

            self.grpcProxy.importFile(
                request, 
                (error: grpc.ServiceError, response: ImportFileResponse) => {
                    if(error) {
                        reject(new Error(error.message));
                    }
                    else {
                        const obj = response.toObject();
                        self.logger.logIpcCall(0, IpcMessageType.Response, "slogger-service.importFile", obj);
                        const result = new ISloggerServerTypes.ImportFileResult();
                        result.resultCode = obj.resultcode
                        resolve(result);
                    }
                });
        });        

    }

    changeQuery(queryRootDirPath: string, queryName: string): Promise<ISloggerServerTypes.ChangeQueryResult> {

        let self = this;

        return new Promise(function(resolve, reject) {

            const request = new QueryRequest();
            request.setQueryname(queryName);
            request.setQueryrootdirpath(queryRootDirPath);

            self.logger.logIpcCall(0, IpcMessageType.Request, "slogger-service.changeQuery", request);

            self.grpcProxy.changeQuery(
                request, 
                (error: grpc.ServiceError, response: QueryResponse) => {
                    if(error) {
                        self.logger.logIpcCall(0, IpcMessageType.Error, "slogger-service.changeQuery", {error: error.message});
                        reject(new Error(error.message));
                    }
                    else {
                        const obj = response.toObject();
                        self.logger.logIpcCall(0, IpcMessageType.Response, "slogger-service.changeQuery", obj);
                        const result = new ISloggerServerTypes.ChangeQueryResult();
                        resolve(result);
                    }
                });    
        });

    }

    getEventsFromTo(fromIndex: number, toIndex: number): Promise<ISloggerServerTypes.GetEventsFromToResult> {
        
        let self = this;

        return new Promise(function(resolve, reject) {

            const request = new GetEventsFromToRequest();
            request.setFromindex(fromIndex);
            request.setToindex(toIndex);

            self.logger.logIpcCall(0, IpcMessageType.Request, "slogger-service.getEventsFromTo", request.toObject());

            self.grpcProxy.getEventsFromTo(
                request, 
                (error: grpc.ServiceError, response: GetEventsFromToResponse) => {
                    if(error) {
                        self.logger.logIpcCall(0, IpcMessageType.Error, "slogger-service.getEventsFromTo", {error: error.message});
                        reject(new Error(error.message));
                    }
                    else {
                        const obj = response.toObject();
                        self.logger.logIpcCall(0, IpcMessageType.Response, "slogger-service.getEventsFromTo", {size: obj.eventsList.length});
                        const result = new ISloggerServerTypes.GetEventsFromToResult();
                        const events = obj.eventsList.map( e  => { 
                            const event = new ISloggerServerTypes.Events();
                            event.rowid = e.rowid;
                            event.timestamp = e.timestamp;
                            event.data = e.dataMap;
                            return event;
                        });
                        result.events = events
                        resolve(result);
                    }
                });    
        });

    }

    clearData(): Promise<ISloggerServerTypes.ClearDataResult> {

        let self = this;

        return new Promise((resolve, reject) => {

            const request = new ClearDataRequest();
            self.logger.logIpcCall(0, IpcMessageType.Request, "slogger-service.clearData", request.toObject());

            self.grpcProxy.clearData(
                request, 
                (error: grpc.ServiceError, response: ClearDataResponse) => {
                    if(error) {
                        self.logger.logIpcCall(0, IpcMessageType.Error, "slogger-service.clearData", {error: error.message});
                        reject(new Error(error.message));
                    }
                    else {
                        const obj = response.toObject();
                        self.logger.logIpcCall(0, IpcMessageType.Response, "slogger-service.clearData", obj);
                        const result = new ISloggerServerTypes.ClearDataResult();
                        resolve(result);
                    }
                });    
        });        

    }
    
    registerOnQueryChanged(): void {

        this.logger.logIpcCall(0, IpcMessageType.Request, "slogger-service.registerOnQueryChanged", {} );

        const call = this.grpcProxy.registerOnQueryChanged(new OnQueryChangedRequest());

        let self = this;

        call.on('data', function(notification: OnQueryChangedNotification) {
            const obj = notification.toObject();
            const result = new ISloggerServerTypes.OnQueryChangedNotification();
            result.resultSize = obj.resultsize;
            result.notifcationData = ISloggerServerTypes.NotifcationData.OK;
            self.logger.logIpcCall(0, IpcMessageType.Notification, "slogger-service.registerOnQueryChanged", result);
            self.client?.onQueryChanged(result);
        });

        call.on('error', function(error: grpc.ServiceError) {
            const result = new ISloggerServerTypes.OnQueryChangedNotification();
            result.notifcationData = ISloggerServerTypes.NotifcationData.ERROR;
            self.logger.logIpcCall(0, IpcMessageType.Notification, "slogger-service.registerOnQueryChanged", result);
            self.client?.onQueryChanged(result);
        });

        call.on('end', function() {
            const result = new ISloggerServerTypes.OnQueryChangedNotification();
            result.notifcationData = ISloggerServerTypes.NotifcationData.STOPPED;
            self.logger.logIpcCall(0, IpcMessageType.Notification, "slogger-service.registerOnQueryChanged", result);
            self.client?.onQueryChanged(result);
        });
    }

    registerOnImportFile(): void {

        this.logger.logIpcCall(0, IpcMessageType.Request, "slogger-service.registerOnImportFile", {} );

        const call = this.grpcProxy.registerOnImportFile(new OnImportFileRequest());

        let self = this;

        call.on('data', function(notification: OnImportFileNotification) {
            const obj = notification.toObject();
            const result = new ISloggerServerTypes.OnImportFileNotification();
            result.numberEvents = obj.numberevents;
            result.distinctFields = obj.distinctfieldsList;
            result.notifcationData = ISloggerServerTypes.NotifcationData.OK;
            self.logger.logIpcCall(0, IpcMessageType.Notification, "slogger-service.registerOnImportFile", result);
            self.client?.onImportFile(result);
        });

        call.on('error', function(error: grpc.ServiceError) {
            const result = new ISloggerServerTypes.OnImportFileNotification();
            result.notifcationData = ISloggerServerTypes.NotifcationData.ERROR;
            self.logger.logIpcCall(0, IpcMessageType.Notification, "slogger-service.registerOnImportFile", result);
            self.client?.onImportFile(result);
        });

        call.on('end', function() {
            const result = new ISloggerServerTypes.OnImportFileNotification();
            result.notifcationData = ISloggerServerTypes.NotifcationData.STOPPED;
            self.logger.logIpcCall(0, IpcMessageType.Notification, "slogger-service.registerOnImportFile", result);
            self.client?.onImportFile(result);
        });
        
    }


    dispose(): void {
        /* no op */
    }

    setClient(client: ISloggerClient | undefined): void {
        console.log("**** SloggerRpcBridge.setClient "+JSON.stringify(client));
        this.client = client;
    }
    
}