import { injectable, inject } from 'inversify';
import { BackendApplicationContribution } from '@theia/core/lib/node';
import { ProcessErrorEvent } from '@theia/process/lib/node/process';
import { ProcessManager } from '@theia/process/lib/node/process-manager';
import { RawProcess, RawProcessFactory } from '@theia/process/lib/node/raw-process';
import * as cp from 'child_process';
import * as express from 'express';
import { TracerNode } from './slogger-log4js';

@injectable()
export class SloggerServiceStarter implements BackendApplicationContribution {

  @inject(ProcessManager) protected readonly processManager: ProcessManager;
  @inject(RawProcessFactory) protected readonly processFactory: RawProcessFactory;
  @inject(TracerNode) protected readonly logger: TracerNode;

  readonly SLOGGER_SERVICE_PATH: string = __dirname+'/../../../../../backend/slogger-service';

  rawProcess: RawProcess;
  readonly launch: boolean = true;

  // /home/ubuntu/tools2/slogger/extensions/core/lib/node

  initialize() {
      if(this.launch)
      {
        this.logger.logState("SloggerServiceStarter.initialize");
        this.logger.logState(this.SLOGGER_SERVICE_PATH);
        this.spawnProcessAsync('bazel', ['run', "--incompatible_strict_action_env", "//slogger_service:slogger_service"], {cwd : this.SLOGGER_SERVICE_PATH});  
      }
  }

  onStop(app?: express.Application)
  {
    if(this.launch)
    {
      this.logger.logState("SloggerServiceStarter.stop");
      this.rawProcess.inputStream.write('q\n');
      this.rawProcess.kill();  
    }
  }

  protected spawnProcessAsync(command: string, args?: string[], options?: cp.SpawnOptions): Promise<RawProcess> {
    this.rawProcess = this.processFactory({ command, args, options });
    this.rawProcess.errorStream.on('data', this.logInfo.bind(this));
    this.rawProcess.outputStream.on('data', this.logInfo.bind(this));

    return new Promise<RawProcess>((resolve, reject) => {
        this.rawProcess.onError((error: ProcessErrorEvent) => {
            this.onDidFailSpawnProcess(error);
            if (error.code === 'ENOENT') {
                const guess = command.split(/\s+/).shift();
                if (guess) {
                    reject(new Error(`Failed to spawn ${guess}\nPerhaps it is not on the PATH.`));
                    return;
                }
            }
            reject(error);
        });
        process.nextTick(() => resolve(this.rawProcess));
      });
    }

    protected onDidFailSpawnProcess(error: Error | ProcessErrorEvent): void {
      this.logger.error(error.message);    
    }

    protected logInfo(data: string | Buffer) {
      if (data) {
          this.logger.info(`SloggerServiceStarter: ${data}`);
      }
    }

}